@@ -24,12 +24,9 @@ from PIL import ImageFont
 from .query_integral_image import query_integral_image
 from .tokenization import unigrams_and_bigrams, process_tokens
 
-item1 = itemgetter(1)
-
-FONT_PATH = os.environ.get("FONT_PATH", os.path.join(os.path.dirname(__file__),
-                                                     "DroidSansMono.ttf"))
-STOPWORDS = set([x.strip() for x in open(
-    os.path.join(os.path.dirname(__file__), 'stopwords')).read().split('\n')])
+FILE = os.path.dirname(__file__)
+FONT_PATH = os.environ.get('FONT_PATH', os.path.join(FILE, 'DroidSansMono.ttf'))
+STOPWORDS = set(map(str.strip, open(os.path.join(FILE, 'stopwords')).readlines()))
 
 
 class IntegralOccupancyMap(object):
@@ -106,7 +103,8 @@ class colormap_color_func(object):
                  random_state=None, **kwargs):
         if random_state is None:
             random_state = Random()
-        r, g, b, _ = 255 * np.array(self.colormap(random_state.uniform(0, 1)))
+        r, g, b, _ = np.maximum(0, 255 * np.array(self.colormap(
+            random_state.uniform(0, 1))))
         return "rgb({:.0f}, {:.0f}, {:.0f})".format(r, g, b)
 
 
@@ -347,7 +345,7 @@ class WordCloud(object):
 
         """
         # make sure frequencies are sorted and normalized
-        frequencies = sorted(frequencies.items(), key=item1, reverse=True)
+        frequencies = sorted(frequencies.items(), key=itemgetter(1), reverse=True)
         if len(frequencies) <= 0:
             raise ValueError("We need at least 1 word to plot a word cloud, "
                              "got %d." % len(frequencies))
@@ -407,8 +405,16 @@ class WordCloud(object):
                                                max_font_size=self.height)
                 # find font sizes
                 sizes = [x[1] for x in self.layout_]
-                font_size = int(2 * sizes[0] * sizes[1] / (sizes[0]
-                                                           + sizes[1]))
+                try:
+                    font_size = int(2 * sizes[0] * sizes[1] 
+                                    / (sizes[0] + sizes[1]))
+                # quick fix for if self.layout_ contains less than 2 values
+                # on very small images it can be empty
+                except IndexError:
+                    try:
+                        font_size = sizes[0]
+                    except IndexError:
+                        raise ValueError('canvas size is too small')
         else:
             font_size = max_font_size
 
@@ -648,7 +654,7 @@ class WordCloud(object):
         """
 
         img = self.to_image()
-        img.save(filename)
+        img.save(filename, optimize=True)
         return self
 
     def to_array(self):
